<?php

namespace App\Http\Controllers\Auth;

use App\KelasTesis;
use App\User;
use App\Manajer;
use App\Dosen;
use App\Mahasiswa;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Foundation\Auth\RegistersUsers;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Auth;
use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;
use App\Imports\UsersImport;

class RegisterController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

    use RegistersUsers;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {

    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array  $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validateRegister(array $data)
    {
        return Validator::make($data, [
            'name' => 'required|string|max:255',
            'username' => 'required|string|max:18|unique:users',
            'email' => 'required|string|email|max:255|unique:users',
            'password' => 'required|string|min:6|confirmed',
            'phone' => 'required|min:8|max:12'
        ]);
    }

    protected function validateUsersRegister(array $data)
    {
        return Validator::make($data, [
            'name' => 'required|string|max:255',
            'username' => 'required|string|max:18|unique:users',
            'email' => 'required|string|email|max:255|unique:users',
            'phone' => 'required|min:8|max:12',
            'password' => 'required|min:6',
        ]);
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array  $data
     * @return \App\User
     */
    protected function create(array $data)
    {
        return User::create([
            'name' => $data['name'],
            'username' => $data['username'],
            'email' => $data['email'],
            'phone' => $data['phone'],
            'password' => Hash::make($data['password']),
        ]);
    }


    public function showForm() {
        if(Auth::user() && Auth::user()->isManajer()) {
            return view('auth.register');
        } else {
            return abort(403);
        }
    }

    public function registerUser(Request $request) {
        $data = $request->all();
        $username = $data['username'];
        $validator = $this->validateRegister($data);

        $alert_success = [];
        $alert_warning = [];

        if ($validator->fails()) {
            $error_messages = json_decode($validator->errors(), true);
            foreach ($error_messages as $error_message) {
                array_push($alert_warning, $error_message[0]);
            }
        } else if (KelasTesis::select('id')->count()==0) {
            $error_kelas_tesis = "Kelas Tesis is not available. Please make sure there is available Kelas Tesis.";
            array_push($alert_warning, $error_kelas_tesis);
        } else {
           $user = $this->create($data);
           $role = $data['role'];
           if($role == User::ROLE_DOSEN) {
               Dosen::create(['id'=>$user->id]);
           }else if($role == User::ROLE_MAHASISWA) {
               $id_kelas_tesis = KelasTesis::orderByRaw('updated_at - created_at DESC')->first();
               Mahasiswa::create(['id'=>$user->id, 'id_kelas_tesis'=>$id_kelas_tesis->id]);
           } else if($role == User::ROLE_MANAJER) {
               Manajer::create(['id'=>$user->id]);
           }
           $success_message = "Success! New user has successfully registered!";
           array_push($alert_success, $success_message);
        }
        Session::flash('alert_success', $alert_success);
        Session::flash('alert_warning', $alert_warning);
        return redirect('/register');
    }

    public function registerUsers(Request $request) {
        $file = $request->file('register_file');
        $extension = $file->getClientOriginalExtension();

        $success = 0;
        $failed = 0;

        $alert_success = [];
        $alert_warning = [];

        if (empty($file)) {
            $file_not_uploaded_message = "File is failed to upload.";
            array_push($alert_warning, $file_not_uploaded_message);
            return redirect('/register');
        } else if (($extension == 'xlsx') || ($extension == 'xls')) {
            $sheets = Excel::toArray(new UsersImport, $file);
            $head_rows = $sheets[0];
            $rows = array_slice($head_rows, 1);

            $kelas_tesis_available = !(KelasTesis::select('id')->count() == 0);
                        
            foreach ($rows as $row) {
                // Buat array index data 
                $data['name'] = $row[0];
                $data['username'] = $row[1];
                $data['email'] = $row[2];
                $data['phone'] = $row[3];
                $data['password'] = $row[4];
                $data['role'] = $row[5];

                $validator = $this->validateUsersRegister($data);
                if ($validator->fails()) {
                    //echo json_encode($validator->errors());
                    $failed += 1;
                } else {
                    $role = $data['role'];
                    if ($role == User::ROLE_MAHASISWA && !($kelas_tesis_available)) {
                        $failed += 1;
                    } else {
                        $success += 1;
                        $user = $this->create($data);
                        if($role == User::ROLE_DOSEN) {
                            Dosen::create(['id'=>$user->id]);
                        } else if($role == User::ROLE_MAHASISWA && ($kelas_tesis_available)) {
                            $id_kelas_tesis = KelasTesis::orderByRaw('updated_at - created_at DESC')->first();
                            Mahasiswa::create(['id'=>$user->id, 'id_kelas_tesis'=>$id_kelas_tesis->id]);
                        } else if($role == User::ROLE_MANAJER) {
                            Manajer::create(['id'=>$user->id]);
                        }
                    }
                }
            }
        }
        if ($success) {
            $success_message = "Success! $success user(s) are added!";
            array_push($alert_success, $success_message);
        }
        if ($failed) {
            $failed_message = "$failed user(s) are failed to add.";
            array_push($alert_warning, $failed_message);
        }
        Session::flash('alert_success', $alert_success);
        Session::flash('alert_warning', $alert_warning);
        return redirect('/register');
    }
}
